import { createEffect, createSignal } from 'solid-js';
import { createCompartmentExtension } from './createCompartmentExtension';
/**
 * Creates a lazy compartment extension for the given CodeMirror EditorView,
 * configured only after the Promise will be resolved.
 **
 * See {@link https://codemirror.net/examples/reconfigure/} for use cases and examples of `Compartments`.
 * Check out {@link https://codemirror.net/docs/ref/#state.Compartment} for more details about Compartment API.
 *
 * @param fn The Promise callback that will return the extension once resolved.
 * @param view The CodeMirror EditorView
 */
export function createLazyCompartmentExtension(fn, view) {
    const [loadedExtension, setLoadedExtension] = createSignal([]);
    const [loading, setLoading] = createSignal(false);
    const reconfigure = createCompartmentExtension(() => loadedExtension(), view);
    const setExtension = (extension) => {
        const resolvedExtension = extension ?? [];
        reconfigure(resolvedExtension);
        setLoadedExtension(resolvedExtension);
    };
    createEffect(() => {
        setLoading(true);
        fn()
            .then((extension) => setExtension(extension))
            .catch(() => setExtension(null))
            .finally(() => setLoading(false));
    });
    return Object.assign(reconfigure, {
        // eslint-disable-next-line solid/reactivity
        get loading() {
            return loading();
        },
    });
}
